/************************************************************************/
/* nieh_forward.c - Netinfoevtd Event Forwarding plugin			*/
/************************************************************************/
/*
 *  (c)2005 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/************************************************************************/



#ifdef WIN32
#include <windows.h>
#endif

#include <stdint.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <signal.h>

#include <cutil/attribute.h>
#include <netinfo/netinfo.h>
#include <netinfo/netinfo_portable.h>
#include <netinfo/event.h>


#define MODULE_NAME             "nieh_forward"
#define MODULE_VERSION          "0.1.0"


#define	CONFIG_DELIMITERS	",\r\n"

#ifndef WIN32
#define	DIR_DELIMITER		"/"
#else
#define	DIR_DELIMITER		"\\"
#endif


struct plugin_data_s {
	unsigned int	flags;

	char		*log_file;
	FILE            *log_fd;

	char		*forward_host;
	char		*forward_port;

	char		*client_cert_file;
	char		*client_key_file;
	char		*client_ca_file;

	EventClient	*ec;

};


/* Global Variables */
struct plugin_data_s plugin_data;



int module_init (char *args)
{
	int		rc;
	char		*argtok;
	char		*argtokp;
	char		*str = NULL;
	Attribute	*attribs = NULL;
	if (args == NULL)
		return NETINFO_RC_INVALIDARG;

	plugin_data.flags = 0;
	plugin_data.log_file = NULL;
	plugin_data.log_fd = NULL;
	plugin_data.forward_host = "localhost";
	plugin_data.forward_port = "5011";
	plugin_data.client_cert_file = NULL;
	plugin_data.client_key_file = NULL;
	plugin_data.client_ca_file = NULL;


	str = strtok_r (args, CONFIG_DELIMITERS, &argtok);
	while (1) {
		if (str == NULL)
			break;
		if (strstr (str, "forward_host=")) {
			str = strtok_r (str, "=", &argtokp);
			plugin_data.forward_host = strdup(strtok_r (NULL,
					CONFIG_DELIMITERS,
					&argtokp));
			attribs = attribute_add (attribs,
						"EventHostname",
						plugin_data.forward_host);
		} else if (strstr (str, "forward_port=")) {
			str = strtok_r (str, "=", &argtokp);
			plugin_data.forward_port = strdup(strtok_r (NULL,
					CONFIG_DELIMITERS,
					&argtokp));
			attribs = attribute_add (attribs,
						"EventPort",
						plugin_data.forward_port);
		} else if (strstr (str, "cert_file=")) {
			str = strtok_r (str, "=", &argtokp);
			plugin_data.client_cert_file = strdup(strtok_r (NULL,
					CONFIG_DELIMITERS,
					&argtokp));
			attribs = attribute_add (attribs,
						"SSLCertificateFile",
						plugin_data.client_cert_file);
		} else if (strstr (str, "key_file=")) {
			str = strtok_r (str, "=", &argtokp);
			plugin_data.client_key_file = strdup(strtok_r (NULL,
					CONFIG_DELIMITERS,
					&argtokp));
			attribs = attribute_add (attribs,
						"SSLPrivateKeyFile",
						plugin_data.client_key_file);
		} else if (strstr (str, "ca_file=")) {
			str = strtok_r (str, "=", &argtokp);
			plugin_data.client_ca_file = strdup(strtok_r (NULL,
					CONFIG_DELIMITERS,
					&argtokp));
			attribs = attribute_add (attribs,
						"SSLTrustedCAFile",
						plugin_data.client_ca_file);
		} else if (strstr (str, "log_file=")) {
			str = strdup(strtok_r (str, "=", &argtokp));
			plugin_data.log_file = strdup(strtok_r (NULL,
					CONFIG_DELIMITERS,
					&argtokp));
		} else { 
			return NETINFO_RC_INVALIDARG;
		}
		str = strtok_r (NULL, CONFIG_DELIMITERS, &argtok);
	}

	if (plugin_data.log_file) {
		plugin_data.log_fd = fopen (plugin_data.log_file, "a");
		if (!plugin_data.log_fd)
			return errno;
	}

	rc = eventclient_init_manual (&plugin_data.ec, attribs, 0);
	if (plugin_data.log_fd)
		(void)fprintf (plugin_data.log_fd, "INFO eventclient_init_manual() rc=%d\n", rc);
	fflush(plugin_data.log_fd);
	if (rc != NETINFO_RC_SUCCESS)
		return rc;

	return NETINFO_RC_SUCCESS;
}


int module_cleanup (void)
{
	int		rc = NETINFO_RC_SUCCESS;
	if (plugin_data.ec != NULL) {
		rc = eventclient_uninit (&plugin_data.ec);
		if (rc != NETINFO_RC_SUCCESS)
			return rc;
	}
	if (plugin_data.log_fd)
		fclose(plugin_data.log_fd);
	if (plugin_data.log_file)
		free (plugin_data.log_file);

	return NETINFO_RC_SUCCESS;
}


int handle_event (EventData *event)
{
	int		rc;

	if (event == NULL)
		return NETINFO_RC_INVALIDARG;

	// Forward the event
	// Internally, the event data is copied onto a queue buffer
	// so we don't need to make a copy of it here.
	rc = eventclient_emit_event (plugin_data.ec, event);
	if (plugin_data.log_fd) {
		(void)fprintf (plugin_data.log_fd, "INFO  handle_event() emit event rc=%d\n", rc);
		fflush(plugin_data.log_fd);
	}
	return rc;
}


