/************************************************************************/
/* sbuffer.c - Simple buffer implementation				*/
/************************************************************************/
/*
 *  (c)2005,2023 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/************************************************************************/


#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <sbuffer.h>

struct _sbuffer_t {
	uint32_t	size;
	uint32_t	reallocs;
	char		*write_pos;
	char		*end_pos;
	char		*buffer;
};


sbuffer_t *sbuffer_new(int size)
{
	sbuffer_t	*sb = NULL;
	sb = calloc(1, sizeof(sbuffer_t));
	if (!sb)
		return NULL;
	sb->size = size;
	sb->reallocs = 0;
	sb->buffer = malloc(size);
	if (!sb->buffer) {
		free(sb);
		return NULL;
	}
	sb->write_pos = sb->buffer;
	sb->end_pos = sb->buffer + size;
	return sb;
}


void sbuffer_free(sbuffer_t *sb)
{
	free(sb->buffer);
	free(sb);
	return;
}


void sbuffer_reset(sbuffer_t *sb)
{
	sb->write_pos = sb->buffer;
	return;
}


int sbuffer_append_string(sbuffer_t *sb, char *str)
{
	int	i = 0;
	char	*wp = sb->write_pos;
	char	*rp = str;
	while ((wp < sb->end_pos) && (*rp != 0)) {
		*wp = *rp;
		wp++;
		rp++;
		i++;
	}
	if ((wp == sb->end_pos) || (*rp != 0)) {
		// the buffer is too small
		return -1;
	}
	sb->write_pos = wp;	
	*sb->write_pos = 0;
	return i;
}


int sbuffer_append_data(sbuffer_t *sb, char *data, int len)
{
	int	i = 0;
	char	*wp = sb->write_pos;
	char	*rp = data;
	while ((wp < sb->end_pos) && (len > 0)) {
		*wp = *rp;
		wp++;
		rp++;
		i++;
		len--;
	}
	if ((wp == sb->end_pos) || (len != 0)) {
		// the buffer is too small
		return -1;
	}
	sb->write_pos = wp;	
	*sb->write_pos = 0;
	return i;
}


void sbuffer_get_readable(sbuffer_t *sb, char **pp, int *plen)
{
	*pp = sb->buffer;
	*plen = sb->write_pos-sb->buffer;
	return;
}



